/*
 *  arch/arm/mach-versatile/include/mach/memory.h
 *
 *  Copyright (C) 2003 ARM Limited
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __ASM_ARCH_MEMORY_H
#define __ASM_ARCH_MEMORY_H

/* Memory overview:

   [ARMcore] <--virtual addr-->
   [ARMmmu] <--physical addr-->
   [GERTmap] <--bus add-->
   [VCperiph]

*/

#define ENABLE_L2_CACHE

/*
 * Physical DRAM offset.
 */
#define PHYS_OFFSET	 (0x00000000)

#ifdef ENABLE_L2_CACHE
#define _REAL_BUS_OFFSET            (0x40000000)   /* Use L2 cache */
#else
#define _REAL_BUS_OFFSET            (0xC0000000)   /* don't use L1 or L2 caches */
#endif

#ifdef CONFIG_512MB_DDR
#define VC_BASE_ADDR_IN_ARM_MAP 0x18000000UL /* offset to the VC physical 0x0 in ARM's view */
#define ARM_BASE_ADDR_IN_VC_MAP_128	0x03000000UL   /* offset of ARM 0x0 address in VC memory map for 128MB SDRAM*/
#define ARM_BASE_ADDR_IN_VC_MAP_256	0x08000000UL   /* offset of ARM 0x0 address in VC memory map for 256MB SDRAM*/
#else
#define VC_BASE_ADDR_IN_ARM_MAP 0x0b800000UL /* offset to the VC physical 0x0 in ARM's view */
#define ARM_BASE_ADDR_IN_VC_MAP_128	0x03000000UL   /* offset of ARM 0x0 address in VC memory map for 128MB SDRAM*/
#define ARM_BASE_ADDR_IN_VC_MAP_256	0x04800000UL   /* offset of ARM 0x0 address in VC memory map for 256MB SDRAM*/
#endif

/* We're using the memory at 64M in the VideoCore for Linux - this adjustment
   will provide the offset into this area as well as setting the bits that
   stop the L1 and L2 cache from being used

   WARNING: this only works because the ARM is given memory at a fixed location
            (ARMMEM_OFFSET)
*/
#ifndef __ASSEMBLY__

#include <mach/ipc.h>

#define __virt_to_bus(x)    __ipc_virt_to_bus(x)
#define __bus_to_virt(x)    __ipc_bus_to_virt(x)

#define __pfn_to_bus(x)     __ipc_pfn_to_bus(x)
#define __bus_to_pfn(x)     __ipc_bus_to_pfn(x)

#endif // #ifndef __ASSEMBLY__
    
/*
 *  Two definitions are required for sparsemem:
 *  
 *  MAX_PHYSMEM_BITS: The number of physical address bits required
 *  to address the last byte of memory.
 *
 *  SECTION_SIZE_BITS: The number of physical address bits to cover
 *  the maximum amount of memory in a section.
 *
 *  Eg, if you have 2 banks of up to 64MB at 0x80000000, 0x84000000,
 *  then MAX_PHYSMEM_BITS is 32, SECTION_SIZE_BITS is 26.
 *            *
 *  Define these in your mach/memory.h.
 */
#ifdef CONFIG_512MB_DDR
#define MAX_PHYSMEM_BITS  29
#else
#define MAX_PHYSMEM_BITS  28
#endif
#define SECTION_SIZE_BITS 23

/*
 * Boost our consistent DMA area to 4M to support 1920x1080 framebuffers
 */

/*
#define CONSISTENT_DMA_SIZE SZ_4M
*/
#endif
