/*
 *  otp_driver.c - otp driver interface
 *
 *  Copyright (C) 2010 Broadcom
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This device provides a shared mechanism for writing to the mailboxes,
 * semaphores, doorbells etc. that are shared between the ARM and the VideoCore
 * processor
 */
#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/errno.h>
#include <asm/uaccess.h>
#include <linux/miscdevice.h>

#include "otp.h"
#include "otp_driver.h"
#include "otp_regs.h"
#include "otp_ioctl.h"

#define OTP_DRIVER_NAME  "otp"
//#define OTP_KBUF


static int otp_opened = 0;

// TODO: proc access has not been implemented.
/*driver functions  */
ssize_t otp_read(struct file *filp, char __user *buf, size_t count,
		loff_t *f_pos)
{
#if 0
	int ret = 0;

	otp_print("-IN buf=%p count=%d\n", buf, count);

	ret = vc_otp_read((uint32_t)(*f_pos), (uint32_t)count, buf);

	otp_print("-OUT ret=%d\n", ret);
	return ret;
#endif
	otp_print("otp_read has not been implemented.\n");
	return 0;
}


ssize_t otp_write(struct file *filp, const char __user *buf, size_t count,
		loff_t *f_pos)
{
#if 0
	int ret = 0;

	otp_print("-IN buf=%p count=%d\n", buf, count);

	/* check the length */
	if( count > (OTP_REQUEST_BUFFER_SIZE - 1) ) {
		printk(KERN_ERR"[%s] max cmd size=%d\n", __func__, OTP_REQUEST_BUFFER_SIZE);
		ret = -ENOMEM;
		goto err_len;
	}

	//	ret = vc_otp_write((uint32_t)(*f_pos), (uint32_t)count, buf);

err_len:
	otp_print("-OUT ret=%d\n", ret);
	return ret;
#endif
	otp_print("otp_read has not been implemented.\n");
	return 0;
}

loff_t otp_seek( struct file *filp, loff_t offset, int mode)
{
#if 0
	if(mode != SEEK_SET)
		return -EINVAL;

	filp->f_pos = offset;
	return 0;
#endif
	otp_print("otp_seek has not been implemented.\n");
	return 0;

}

int otp_open (struct inode *inode, struct file *filp)
{
	int ret = 0;

	otp_print("-IN\n");

    // Only allow 1 user
    if (otp_opened) {
        return -EBUSY;
    }
    otp_opened++;

	otp_print("-OUT ret=%d\n", ret);
	return ret;
}

// allow read/write of otp, but restricting to known otp registers/rows available
int otp_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
		unsigned long arg)
{
	int ret = 0;
	struct OTP_IOCTL_DATA_CMD command;

	ret = copy_from_user(&command, (void*)arg, sizeof(struct OTP_IOCTL_DATA_CMD));
	if(ret != 0) {
		printk(KERN_ERR "[%s]error in copying cmd structure\n", __func__);
		return -EIO;
	}

	switch(cmd){
		// write
		case OTP_IOCTL_WRITE:
#if 0
			switch(otp_ioctl_arg.otp_row) {
				// 4 rows sized
				case OTP_JTAG_DEBUG_KEY_ROW:
				case OTP_JTAG_DEBUG_KEY_ROW_REDUNDANT:
				case OTP_VPU_CACHE_KEY_ROW:
				case OTP_VPU_CACHE_KEY_ROW_REDUNDANT:
				case OTP_JTAG_VPU_PARITY_REDUNDANT:
				case OTP_BOOT_SIGNING_KEY_ROW:
				case OTP_BOOT_SIGNING_KEY_ROW_REDUNDANT:
				case OTP_CODE_SIGNING_KEY_ROW:
				case OTP_CODE_SIGNING_KEY_ROW_REDUNDANT:
				case OTP_HDCP_AES_KEY_ROW:
				case OTP_HDCP_AES_KEY_ROW_REDUNDANT:
				case OTP_PUBLIC_KEY_ROW:
				case OTP_PUBLIC_KEY_ROW_REDUNDANT:
				case OTP_PRIVATE_KEY_ROW:
				case OTP_PRIVATE_KEY_ROW_REDUNDANT:

					if(otp_ioctl_arg.length !=4)
						return -EINVAL;

					ret = vc_otp_write(otp_ioctl_arg.otp_row, 4, otp_ioctl_arg.data);

					break;

					// 1 row sized
				case OTP_CONTROL_ROW:
				case OTP_BOOT_ROM_ROW:
				case OTP_BOOT_ROM_ROW_REDUNDANT:
				case OTP_BOOT_SIGNING_PARITY_ROW:
				case OTP_CODE_SIGNING_PARITY_ROW:
				case OTP_HDCP_AES_PARITY_ROW:
				case OTP_PUBLIC_PARITY_ROW:
				case OTP_PRIVATE_PARITY_ROW:
				case OTP_CODE_SIGNING_FLAG_ROW :
					if(otp_ioctl_arg.length !=1)
						return -EINVAL;

					ret = vc_otp_write(otp_ioctl_arg.otp_row, 1, otp_ioctl_arg.data);

					break;
			}

			break;
#endif
		case OTP_IOCTL_READ:
#if 0
			// read
			// user allocated a buffer to receive requested data
			switch(otp_ioctl_arg.otp_row) {
				// 4 rows sized
				case OTP_JTAG_DEBUG_KEY_ROW:
				case OTP_JTAG_DEBUG_KEY_ROW_REDUNDANT:
				case OTP_VPU_CACHE_KEY_ROW:
				case OTP_VPU_CACHE_KEY_ROW_REDUNDANT:
				case OTP_JTAG_VPU_PARITY_REDUNDANT:
				case OTP_BOOT_SIGNING_KEY_ROW: 
				case OTP_BOOT_SIGNING_KEY_ROW_REDUNDANT:
				case OTP_CODE_SIGNING_KEY_ROW:
				case OTP_CODE_SIGNING_KEY_ROW_REDUNDANT:
				case OTP_HDCP_AES_KEY_ROW:
				case OTP_HDCP_AES_KEY_ROW_REDUNDANT:
				case OTP_PUBLIC_KEY_ROW:
				case OTP_PUBLIC_KEY_ROW_REDUNDANT:
				case OTP_PRIVATE_KEY_ROW:
				case OTP_PRIVATE_KEY_ROW_REDUNDANT:
					if(otp_ioctl_arg.length !=4)
						return -EINVAL;

					ret = vc_otp_read(otp_ioctl_arg.otp_row, 4, otp_ioctl_arg.data);
					break;

					// 1 row sized
				case OTP_CONTROL_ROW:
				case OTP_BOOT_ROM_ROW:
				case OTP_BOOT_ROM_ROW_REDUNDANT:
				case OTP_BOOT_SIGNING_PARITY_ROW:
				case OTP_CODE_SIGNING_PARITY_ROW:
				case OTP_HDCP_AES_PARITY_ROW:
				case OTP_PUBLIC_PARITY_ROW:
				case OTP_PRIVATE_PARITY_ROW:
				case OTP_CODE_SIGNING_FLAG_ROW :
					if(otp_ioctl_arg.length !=1)
						return -EINVAL;

					ret = vc_otp_read(otp_ioctl_arg.otp_row, 1, otp_ioctl_arg.data);

					break;
			}
#endif
			printk("Not implemented, use _RAW version.\n");
			break;
	    case OTP_IOCTL_WRITE_RAW:
		    if ((command.length * 4) > OTP_REQUEST_BUFFER_SIZE) {
				ret = -ENOMEM;
				break;
			} else if (command.length == 0) {
				break;
			}
			if (ret < 0) {
				printk(KERN_ERR"[%s] error in getting buffer semaphore\n", __func__ );
				break;
			}

            // Function does copy_from_user
			ret = vc_otp_write(command.otp_addr, command.length, (char *)command.data);
			break;
		case OTP_IOCTL_READ_RAW:
			if ((command.length * 4) > OTP_REQUEST_BUFFER_SIZE) {
				ret = -ENOMEM;
				break;
			} else if (command.length == 0) {
				break;
			}

            // Function does copy_to_user
			ret = vc_otp_read(command.otp_addr, command.length, (char *)command.data);
			if (ret) {
				printk("Error in otp_read");
			}
			break;

		case OTP_IOCTL_TEST_ENABLE:
			vc_otp_test(1);
			break;

		case OTP_IOCTL_TEST_DISABLE:
			vc_otp_test(0);
			break;

		case OTP_IOCTL_VERIFY_RESULTS:
			if ((command.length * 4) > OTP_REQUEST_BUFFER_SIZE) {
				ret = -ENOMEM;
				break;
			} else if (command.length == 0) {
				ret = -EINVAL;
				break;
			}

            // Function does copy_to_user
			ret = vc_otp_verify_results(command.otp_addr, command.length, (char *)command.data);
			if (ret) {
				printk("Error in otp_verify_results");
			}
			break;
	}

	return ret;
}


int otp_release(struct inode *inode, struct file *filp)
{
	int ret = 0;
	otp_print("-IN");

    otp_opened--;

	otp_print("-OUT ret=%d\n", ret);
	return ret;
}

struct file_operations otp_fops = {
	.owner =	THIS_MODULE,
	.read =		otp_read,
	.write =	otp_write,
	.llseek =	otp_seek,
	.ioctl =	otp_ioctl,
	.open =		otp_open,
	.release =	otp_release,
};

struct miscdevice otp_dev = {
	.minor =    MISC_DYNAMIC_MINOR,
	.name =     OTP_DRIVER_NAME,
	.fops =     &otp_fops
};


int __devinit otp_driver_init(void)
{
	int ret;

	ret = misc_register(&otp_dev);
	if (ret < 0) {
		printk(KERN_ERR "%s:failed to register device\n",
				OTP_DRIVER_NAME);
	}

	return ret;
}

void __devexit otp_driver_exit(void)
{
	misc_deregister(&otp_dev);
}
