
#ifndef _OTP_IOCTL_H
#define _OTP_IOCTL_H

enum {
    OTP_IOCTL_ID_READ = 0x81,
    OTP_IOCTL_ID_READ_RAW = 0x82,
    OTP_IOCTL_ID_WRITE = 0x83,
    OTP_IOCTL_ID_WRITE_RAW = 0x84,
    OTP_IOCTL_ID_TEST_ENABLE = 0x85,
    OTP_IOCTL_ID_TEST_DISABLE = 0x86,
    OTP_IOCTL_ID_VERIFY_RESULTS = 0x87,
} OTP_COMMAND_ID_T;

#define OTP_MAGIC 'O'

struct OTP_IOCTL_DATA_CMD {
  uint32_t otp_addr; // as listed below and in hardware_vc4.h
  uint32_t length;
  uint32_t *data;
};

#define OTP_IOCTL_READ          _IOR(OTP_MAGIC, OTP_IOCTL_ID_READ, struct OTP_IOCTL_DATA_CMD)
#define OTP_IOCTL_READ_RAW      _IOR(OTP_MAGIC, OTP_IOCTL_ID_READ_RAW, struct OTP_IOCTL_DATA_CMD)
#define OTP_IOCTL_WRITE         _IOR(OTP_MAGIC, OTP_IOCTL_ID_WRITE, struct OTP_IOCTL_DATA_CMD)
#define OTP_IOCTL_WRITE_RAW     _IOR(OTP_MAGIC, OTP_IOCTL_ID_WRITE_RAW, struct OTP_IOCTL_DATA_CMD)
#define OTP_IOCTL_TEST_ENABLE     _IOR(OTP_MAGIC, OTP_IOCTL_ID_TEST_ENABLE, struct OTP_IOCTL_DATA_CMD)
#define OTP_IOCTL_TEST_DISABLE     _IOR(OTP_MAGIC, OTP_IOCTL_ID_TEST_DISABLE, struct OTP_IOCTL_DATA_CMD)
#define OTP_IOCTL_VERIFY_RESULTS     _IOR(OTP_MAGIC, OTP_IOCTL_ID_VERIFY_RESULTS, struct OTP_IOCTL_DATA_CMD)

//OTP bit definitions
// NB the addressing has changed between VCIII and VCIV - we now address by row (each row contains 32 bits)
#define OTP_JTAG_DEBUG_KEY_ROW                8
#define OTP_JTAG_DEBUG_KEY_SIZE_IN_ROWS       4
#define OTP_VPU_CACHE_KEY_ROW                 (OTP_JTAG_DEBUG_KEY_ROW+OTP_JTAG_DEBUG_KEY_SIZE_IN_ROWS)
#define OTP_VPU_CACHE_KEY_SIZE_IN_ROWS        4

#define OTP_CONTROL_ROW                       (OTP_VPU_CACHE_KEY_ROW+OTP_VPU_CACHE_KEY_SIZE_IN_ROWS)
#define OTP_CONTROL_SIZE_IN_ROWS              1

#define OTP_BOOT_ROM_ROW                      (OTP_CONTROL_ROW+OTP_CONTROL_SIZE_IN_ROWS)
#define OTP_BOOT_ROM_SIZE_IN_ROWS             1

#define OTP_BOOT_ROM_ROW_REDUNDANT            (OTP_BOOT_ROM_ROW+OTP_BOOT_ROM_SIZE_IN_ROWS)

#ifdef __BCM2708A0__
   #define OTP_BOOT_SIGNING_KEY_ROW              (OTP_BOOT_ROM_ROW_REDUNDANT+OTP_BOOT_ROM_SIZE_IN_ROWS)
   #define OTP_BOOT_SIGNING_KEY_SIZE_IN_ROWS     4

   #define OTP_BOOT_SIGNING_PARITY_ROW           (OTP_BOOT_SIGNING_KEY_ROW+OTP_BOOT_SIGNING_KEY_SIZE_IN_ROWS)
   #define OTP_BOOT_SIGNING_PARITY_SIZE_IN_ROWS  1

   #define OTP_CODE_SIGNING_KEY_ROW              (OTP_BOOT_SIGNING_PARITY_ROW+OTP_BOOT_SIGNING_PARITY_SIZE_IN_ROWS)
   #define OTP_CODE_SIGNING_KEY_SIZE_IN_ROWS     4

   #define OTP_CODE_SIGNING_PARITY_ROW           (OTP_CODE_SIGNING_KEY_ROW+OTP_CODE_SIGNING_KEY_SIZE_IN_ROWS)
   #define OTP_CODE_SIGNING_PARITY_SIZE_IN_ROWS  1

   #define OTP_HDCP_AES_KEY_ROW                  (OTP_CODE_SIGNING_PARITY_ROW+OTP_CODE_SIGNING_PARITY_SIZE_IN_ROWS)
   #define OTP_HDCP_AES_KEY_SIZE_IN_ROWS         4

   #define OTP_HDCP_AES_PARITY_ROW               (OTP_HDCP_AES_KEY_ROW+OTP_HDCP_AES_KEY_SIZE_IN_ROWS)
   #define OTP_HDCP_AES_PARITY_SIZE_IN_ROWS      1

   #define OTP_PUBLIC_KEY_ROW                    (OTP_HDCP_AES_PARITY_ROW+OTP_HDCP_AES_PARITY_SIZE_IN_ROWS)
   #define OTP_PUBLIC_KEY_SIZE_IN_ROWS           4

   #define OTP_PUBLIC_PARITY_ROW                 (OTP_PUBLIC_KEY_ROW+OTP_PUBLIC_KEY_SIZE_IN_ROWS)
   #define OTP_PUBLIC_PARITY_SIZE_IN_ROWS        1

   #define OTP_PRIVATE_KEY_ROW                   (OTP_PUBLIC_PARITY_ROW+OTP_PUBLIC_PARITY_SIZE_IN_ROWS)
   #define OTP_PRIVATE_KEY_SIZE_IN_ROWS          4

   #define OTP_PRIVATE_PARITY_ROW                (OTP_PRIVATE_KEY_ROW+OTP_PRIVATE_KEY_SIZE_IN_ROWS)
   #define OTP_PRIVATE_PARITY_SIZE_IN_ROWS       1

   #define OTP_CODE_SIGNING_FLAG_ROW             (OTP_PRIVATE_PARITY_ROW+OTP_PRIVATE_PARITY_SIZE_IN_ROWS)
   #define OTP_CODE_SIGNING_FLAG_SIZE_IN_ROWS    1
#else
   // BCM2708B0 has less reliable OTP so we need a redundant row for the boot signing key (and the corresponding parity)
   #define OTP_BOOT_SIGNING_KEY_ROW              (OTP_BOOT_ROM_ROW_REDUNDANT+OTP_BOOT_ROM_SIZE_IN_ROWS)                 // 19
   #define OTP_BOOT_SIGNING_KEY_SIZE_IN_ROWS     4

   #define OTP_BOOT_SIGNING_KEY_ROW_REDUNDANT    (OTP_BOOT_SIGNING_KEY_ROW+OTP_BOOT_SIGNING_KEY_SIZE_IN_ROWS)           // 23

   #define OTP_BOOT_SIGNING_PARITY_ROW           (OTP_BOOT_SIGNING_KEY_ROW_REDUNDANT+OTP_BOOT_SIGNING_KEY_SIZE_IN_ROWS) // 27
   #define OTP_BOOT_SIGNING_PARITY_SIZE_IN_ROWS  1

   #define OTP_CODE_SIGNING_KEY_ROW              (OTP_BOOT_SIGNING_PARITY_ROW+OTP_BOOT_SIGNING_PARITY_SIZE_IN_ROWS)     // 28
   #define OTP_CODE_SIGNING_KEY_SIZE_IN_ROWS     4

   #define OTP_CODE_SIGNING_KEY_ROW_REDUNDANT    (OTP_CODE_SIGNING_KEY_ROW+OTP_CODE_SIGNING_KEY_SIZE_IN_ROWS)           // 32

   #define OTP_CODE_SIGNING_PARITY_ROW           (OTP_CODE_SIGNING_KEY_ROW_REDUNDANT+OTP_CODE_SIGNING_KEY_SIZE_IN_ROWS) // 36
   #define OTP_CODE_SIGNING_PARITY_SIZE_IN_ROWS  1

   #define OTP_HDCP_AES_KEY_ROW                  (OTP_CODE_SIGNING_PARITY_ROW+OTP_CODE_SIGNING_PARITY_SIZE_IN_ROWS)     // 37
   #define OTP_HDCP_AES_KEY_SIZE_IN_ROWS         4

   #define OTP_HDCP_AES_KEY_ROW_REDUNDANT        (OTP_HDCP_AES_KEY_ROW+OTP_HDCP_AES_KEY_SIZE_IN_ROWS)                   // 41

   #define OTP_HDCP_AES_PARITY_ROW               (OTP_HDCP_AES_KEY_ROW_REDUNDANT+OTP_HDCP_AES_KEY_SIZE_IN_ROWS)         // 45
   #define OTP_HDCP_AES_PARITY_SIZE_IN_ROWS      1

   #define OTP_PUBLIC_KEY_ROW                    (OTP_HDCP_AES_PARITY_ROW+OTP_HDCP_AES_PARITY_SIZE_IN_ROWS)             // 46
   #define OTP_PUBLIC_KEY_SIZE_IN_ROWS           4

   #define OTP_PUBLIC_KEY_ROW_REDUNDANT          (OTP_PUBLIC_KEY_ROW+OTP_PUBLIC_KEY_SIZE_IN_ROWS)                       // 50

   #define OTP_PUBLIC_PARITY_ROW                 (OTP_PUBLIC_KEY_ROW_REDUNDANT+OTP_PUBLIC_KEY_SIZE_IN_ROWS)             // 54
   #define OTP_PUBLIC_PARITY_SIZE_IN_ROWS        1

   #define OTP_PRIVATE_KEY_ROW                   (OTP_PUBLIC_PARITY_ROW+OTP_PUBLIC_PARITY_SIZE_IN_ROWS)                 // 55
   #define OTP_PRIVATE_KEY_SIZE_IN_ROWS          4

   #define OTP_PRIVATE_KEY_ROW_REDUNDANT         (OTP_PRIVATE_KEY_ROW+OTP_PRIVATE_KEY_SIZE_IN_ROWS)                     // 59

   #define OTP_PRIVATE_PARITY_ROW                (OTP_PRIVATE_KEY_ROW_REDUNDANT+OTP_PRIVATE_KEY_SIZE_IN_ROWS)           // 63
   #define OTP_PRIVATE_PARITY_SIZE_IN_ROWS       1

   #define OTP_CODE_SIGNING_FLAG_ROW             (OTP_PRIVATE_PARITY_ROW+OTP_PRIVATE_PARITY_SIZE_IN_ROWS)               // 64
   #define OTP_CODE_SIGNING_FLAG_SIZE_IN_ROWS    1

   // locations fixed by hardware
   #define OTP_JTAG_DEBUG_KEY_ROW_REDUNDANT      68
   #define OTP_VPU_CACHE_KEY_ROW_REDUNDANT       72
   #define OTP_JTAG_VPU_PARITY_REDUNDANT         76

   #if( OTP_CODE_SIGNING_FLAG_ROW+OTP_CODE_SIGNING_FLAG_SIZE_IN_ROWS >= OTP_JTAG_DEBUG_KEY_ROW_REDUNDANT )
      #error "User OTP space has overwritten CPU bits" OTP_CODE_SIGNING_FLAG_ROW OTP_CODE_SIGNING_FLAG_SIZE_IN_ROWS OTP_JTAG_DEBUG_KEY_ROW_REDUNDANT
   #endif

#endif

#define OTP_MIN_ROW                           OTP_JTAG_DEBUG_KEY_ROW
#define OTP_MAX_ROW                           OTP_CODE_SIGNING_FLAG_ROW

// bit locations in the CONTROL OTP register
#define OTP_JTAG_DEBUG_KEY_PARITY_START_BIT   0
#define OTP_VPU_CACHE_KEY_PARITY_START_BIT    8
#define OTP_JTAG_DISABLE_BIT                  16
#define OTP_JTAG_DISABLE_REDUNDANT_BIT        17
#define OTP_MACROVISION_START_BIT             18
#define OTP_MACROVISION_REDUNDANT_START_BIT   20
#define OTP_DECRYPTION_ENABLE_FOR_DEBUG       22
#define OTP_ARM_DISABLE_BIT                   24
#define OTP_ARM_DISABLE_REDUNDANT_BIT         25

#define OTP_JTAG_PARITY_MASK                  0xFF
#define OTP_VPU_CACHE_PARITY_MASK             0xFF

#define OTP_BYTES_PER_ROW                     4

// make sure locations used by the hardware are correct
#if( OTP_CONTROL_ROW != 16)
   #error "The OTP control row has moved - it must be 16!"
#endif
#if( OTP_BOOT_ROM_ROW != 17 )
   #error "The OTP bootrom row has moved - it must be 17!"
#endif
#if( OTP_BOOT_ROM_ROW_REDUNDANT != 18 )
   #error "The OTP bootrom copy row has moved - it must be 18!"
#endif


#endif //_OTP_IOCTL_H
