#ifndef __BCM2708_FB_H
#define __BCM2708_FB_H

#include <asm/pgtable.h>

/* Macros to set pgprot values to match write-through
 * and write-back cache bits for userspace mapping
 * of framebuffer
 **/

#define pgprot_writethrough(prot) \
	__pgprot((pgprot_val(prot) & ~L_PTE_MT_MASK) | L_PTE_MT_WRITETHROUGH)

#define pgprot_writeback(prot) \
	__pgprot((pgprot_val(prot) & ~L_PTE_MT_MASK) | L_PTE_MT_WRITEBACK)

#ifndef FBIO_WAITFORVSYNC
#define FBIO_WAITFORVSYNC	_IOW('F', 0x20, __u32)
#endif

/* macros */
#define MAKEFOURCC(ch0, ch1, ch2, ch3) ((unsigned int)(unsigned char)(ch0) \
		| ((unsigned int)(unsigned char)(ch1) << 8) \
		| ((unsigned int)(unsigned char)(ch2) << 16) \
		| ((unsigned int)(unsigned char)(ch3) << 24 ))

#define FBUF_FOURCC	MAKEFOURCC('F', 'B', 'U', 'F')

#define FB_DEV_NAME_SIZE_MAX		(256)

/*
 * Uncomment this define to include arm update count
 * in control register for verification on VC side.
 *
 * This is useful when debugging lockups, VC will assert
 * if ARM and VC update counts dont match
 */

/*
 * #define DEBUG_UPDATE_COUNT	1
 */

/* Uncomment this to see bcm2708fb_debug() messages */
/*
 * #define BCM2708FB_DEBUG_ENABLE	1
 */


#ifdef BCM2708FB_DEBUG_ENABLE
#define bcm2708fb_debug(fmt, arg...)	\
	printk(KERN_INFO"%s:%d " fmt, __func__, __LINE__, ##arg)
#else
#define bcm2708fb_debug(fmt, arg...)	do {} while(0)
#endif

#define bcm2708fb_info(fmt, arg...)	\
	printk(KERN_INFO"%s:%d " fmt, __func__, __LINE__, ##arg)

#define bcm2708fb_error(fmt, arg...)	\
	printk(KERN_ERR"%s:%d " fmt, __func__, __LINE__, ##arg)

#define bcm2708fb_warning(fmt, arg...)	\
	printk(KERN_WARNING"%s:%d " fmt, __func__, __LINE__, ##arg)

#define bcm2708fb_alert(fmt, arg...)	\
	printk(KERN_ALERT"%s:%d " fmt, __func__, __LINE__, ##arg)

#define FB_CONTROL			(0x00)
	#define	CTRL_ENABLE_SHIFT	(0)
	#define CTRL_ENABLE_MASK	(0x00000001)
	#define CTRL_DISP_EN_SHIFT	(1)
	#define CTRL_DISP_EN_MASK	(0x00000002)
	#define CTRL_UPDT_EN_SHIFT	(2)
	#define CTRL_UPDT_EN_MASK	(0x00000004)
	#define CTRL_UPDT_CNT_DBG_SHIFT	(3)
	#define CTRL_UPDT_CNT_DBG_MASK	(0x00000008)
	#define CTRL_BUFF_IDX_SHIFT	(8)
	#define CTRL_BUFF_IDX_MASK	(0x00000f00)
	#define	CTRL_UPDATE_COUNT_SHIFT	(16)
	#define CTRL_UPDATE_COUNT_MASK	(0x00ff0000)

#define FB_STATUS			(0x04)	/* read only */
	#define STAT_EN_SHIFT		(0)
	#define STAT_EN_MASK		(0x00000001)
	#define STAT_DISP_SHIFT		(1)
	#define STAT_DISP_MASK		(0x00000002)
	#define STAT_UPDT_EN_SHIFT	(2)
	#define STAT_UPDT_EN_MASK	(0x00000004)
	#define STAT_BUFF_IDX_SHIFT	(8)
	#define STAT_BUFF_IDX_MASK	(0x00000f00)
	#define	STAT_UPDATE_COUNT_SHIFT	(16)
	#define STAT_UPDATE_COUNT_MASK	(0x00ff0000)

#define FB_WIDTH			(0x20)	/* Should be read only?? */

#define FB_HEIGHT			(0x24)	/* Should be read only?? */

#define FB_SET_FORMAT			(0x28)
	#define PIXEL_FORMAT_SHIFT	(0)
	#define PIXEL_FORMAT_MASK	(0x000000ff)
	#define FB_FORMAT_RGB565	(0)
	#define FB_FORMAT_RGB888	(1)
	#define FB_FORMAT_RGBA32	(2)
	#define FB_FORMAT_BGR565	(3)
	#define FB_FORMAT_BGR888	(4)
	#define FB_FORMAT_YUVNV12	(5)
	#define FB_FORMAT_YUVNV21	(6)

	#define COLOUR_SPACE_SHIFT	(16)
	#define COLOUR_SPACE_MASK	(0xffff0000)
	#define COLOUR_SPACE_601	(1)
	#define COLOUR_SPACE_709	(2)
	#define COLOUR_SPACE_JPG	(3)

#define FB_SET_NUM_BUFFERS		(0x2C)

#define FB_SET_DISPLAY			(0x40)
#define FB_TARGET_LAYER			(0x44)

/* Target coordinates and size */
#define FB_TARGET_X			(0x48)
#define FB_TARGET_Y			(0x4C)
#define FB_TARGET_WIDTH			(0x50)
#define FB_TARGET_HEIGHT		(0x54)

#define FB_SET_OPACITY			(0x58)

/* Source coordinates and size */
#define FB_SRC_X			(0x60)
#define FB_SRC_Y			(0x64)
#define FB_SRC_WIDTH			(0x68)
#define FB_SRC_HEIGHT			(0x6C)

#define FB_GET_BUFFER_PHYS		(0x70)	/* read only */

/* Register values used by the driver .. */
#define FRAMEBUFFER_TARGET_DISPLAY	(2)
#define FRAMEBUFFER_WIDTH		(1280)
#define FRAMEBUFFER_HEIGHT		(720)
#define FRAMEBUFFER_NUM_BUFFERS		(2)
#define OVERLAY_NUM_BUFFERS		(1)


#endif /* __BCM2708_FB_H */
