#ifndef _LINUX_COREDUMP_H
#define _LINUX_COREDUMP_H

#include <linux/types.h>
#include <linux/mm.h>
#include <linux/fs.h>

#if 1 // CONFIG_LITTLE_CORE_DUMP
#define LTCORE_MAX_PAGES 256
extern int little_core_dump;
extern unsigned ltcore_dump_cnt;
extern char ltcore_dump_file[];
#endif

/*
 * These are the only things you should do on a core-file: use only these
 * functions to write out all the necessary info.
 */
static inline int dump_write(struct file *file, const void *addr, int nr)
{
#if 1 // CONFIG_LITTLE_CORE_DUMP
	if (!little_core_dump)
#endif
		return file->f_op->write(file, addr, nr, &file->f_pos) == nr;

#if 1 // CONFIG_LITTLE_CORE_DUMP
	while (nr) {
		unsigned page = ltcore_dump_cnt >> PAGE_SHIFT;
		unsigned offset = ltcore_dump_cnt & ~PAGE_MASK;
		unsigned chnk = PAGE_SIZE - offset;
		if (nr < chnk) chnk = nr;
		if (ltcore_dump_cnt >= LTCORE_MAX_PAGES*PAGE_SIZE)
			return 0;
		memcpy(ltcore_dump_file + page*PAGE_SIZE + offset, addr, chnk);
		ltcore_dump_cnt += chnk;
		//tracek("ltcore_dump_cnt: %d", ltcore_dump_cnt, 0);
		nr -= chnk;
		addr = (char*)addr + chnk;
	}
	return 1;
#endif
}

static inline int dump_seek(struct file *file, loff_t off)
{
	int ret = 1;

	if (
#if 1 // CONFIG_LITTLE_CORE_DUMP
		!little_core_dump &&
#endif
		file->f_op->llseek && file->f_op->llseek != no_llseek) {
		if (file->f_op->llseek(file, off, SEEK_CUR) < 0)
			return 0;
	} else {
		char *buf = (char *)get_zeroed_page(GFP_KERNEL);

		if (!buf)
			return 0;
		while (off > 0) {
			unsigned long n = off;

			if (n > PAGE_SIZE)
				n = PAGE_SIZE;
			if (!dump_write(file, buf, n)) {
				ret = 0;
				break;
			}
			off -= n;
		}
		free_page((unsigned long)buf);
	}
	return ret;
}

#endif /* _LINUX_COREDUMP_H */
